/*
 * $Id: AbstractResolver.java 286 2005-12-03 01:07:16Z davidb $
 * 
 * Copyright (c) 2005 VeriSign. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer. 2. Redistributions in
 * binary form must reproduce the above copyright notice, this list of
 * conditions and the following disclaimer in the documentation and/or other
 * materials provided with the distribution. 3. The name of the author may not
 * be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *  
 */

package se.rfc.unbound;

import java.util.Properties;

/**
 * This is an abstract base class for resolver modules. This base class
 * basically takes care of propagating actions to the next module in the
 * chain.
 * 
 * @author davidb
 * @version $Revision: 286 $
 */
public abstract class AbstractResolver implements Resolver
{

  protected Resolver mNextResolver;
  protected Resolver mPrevResolver;
  protected Resolver mHeadResolver;

  /**
   * This is a basic constructor for a resolver module. It takes a reference
   * to the next resolver module in the chain of resolver modules.
   * 
   * @param nextResolver The next resolver module in the chain.
   */
  protected AbstractResolver(Resolver nextResolver)
  {
    this.mNextResolver = nextResolver;
    if (mNextResolver != null)
    {
      mNextResolver.setPrevResolver(this);
    }
  }

  /**
   * Initialize the module from the included configuration. This method will
   * propagate down the chain.
   * 
   * @param config The configuration.
   */
  public void init(Properties config) throws Exception
  {
    if (mNextResolver != null) mNextResolver.init(config);
  }

  /**
   * Prepare the module for exiting. This method will propagate down the
   * chain.
   */
  public void shutdown()
  {
    if (mNextResolver != null) mNextResolver.shutdown();
  }

  /**
   * Process a request event -- request events move from the head of the chain
   * to the tail.
   * 
   * @param event The event to process.
   */
  public abstract void processRequest(DNSEvent event);

  /**
   * Process a response event -- response events from from the tail toward the
   * head.
   * 
   * @param event The event to process.
   */
  public abstract void processResponse(DNSEvent event);

  /**
   * Get a reference to the next module in the chain. (Next is towards the
   * tail of the chain.)
   */
  public Resolver getNextResolver()
  {
    return mNextResolver;
  }

  /**
   * Get a reference tot he previous module in the chain. (Previous is towards
   * the head of the chain.)
   */
  public Resolver getPrevResolver()
  {
    return mPrevResolver;
  }

  /**
   * Set (or reset) the previous resolver.
   */
  public void setPrevResolver(Resolver prev)
  {
    mPrevResolver = prev;
  }

  /**
   * Set the resultion chain head -- each component in the chain that might
   * wish to internally generate a new query will probably want to inject that
   * query at the head of the resolution chain.
   */
  public void setResolutionHead(Resolver resolver)
  {
    mHeadResolver = resolver;
    if (mNextResolver != null) mNextResolver.setResolutionHead(resolver);
  }

  /**
   * @return The head of the resolution chain. This is the preferred place to
   *         inject new, internally generated, DNS queries.
   */
  protected Resolver getResolutionHead()
  {
    return mHeadResolver;
  }
}